@@ -4,7 +4,7 @@
 
 ;; Author: Syohei YOSHIDA <syohex@gmail.com>
 ;; URL: https://github.com/syohex/emacs-helm-ag
-;; Version: 0.58
+;; Version: 0.59
 ;; Package-Requires: ((emacs "24.4") (helm "2.0"))
 
 ;; This program is free software; you can redistribute it and/or modify
@@ -48,65 +48,78 @@
   :group 'helm)
 
 (defsubst helm-ag--windows-p ()
+  "Check ag available window."
   (memq system-type '(ms-dos windows-nt)))
 
 (defcustom helm-ag-base-command
   (if (helm-ag--windows-p)
       "ag --vimgrep"
     "ag --nocolor --nogroup")
-  "Base command of `ag'"
-  :type 'string)
+  "Base command of `ag'."
+  :type 'string
+  :group 'helm-ag)
 
 (defcustom helm-ag-command-option nil
-  "Command line option of `ag'. This is appended after `helm-ag-base-command'"
-  :type 'string)
+  "Command line option of `ag'.  This is appended after `helm-ag-base-command'."
+  :type 'string
+  :group 'helm-ag)
 
 (defcustom helm-ag-insert-at-point nil
   "Insert thing at point as search pattern.
-   You can set value same as `thing-at-point'"
-  :type 'symbol)
+You can set value same as `thing-at-point'"
+  :type 'symbol
+  :group 'helm-ag)
 
 (defcustom helm-ag-ignore-patterns nil
-  "Ignore patterns for `ag'. This parameters are specified as --ignore"
+  "Ignore patterns for `ag'.  This parameters are specified as --ignore."
   :type '(repeat string))
 
 (defcustom helm-ag-use-grep-ignore-list nil
   "Use `grep-find-ignored-files' and `grep-find-ignored-directories' as ignore pattern.
 They are specified to `--ignore' options."
-  :type 'boolean)
+  :type 'boolean
+  :group 'helm-ag)
 
 (defcustom helm-ag-always-set-extra-option nil
   "Always set `ag' options of `helm-do-ag'."
-  :type 'boolean)
+  :type 'boolean
+  :group 'helm-ag)
 
 (defcustom helm-ag-fuzzy-match nil
   "Enable fuzzy match"
-  :type 'boolean)
+  :type 'boolean
+  :group 'helm-ag)
 
 (defcustom helm-ag-edit-save t
   "Save buffers you edit at completed."
-  :type 'boolean)
+  :type 'boolean
+  :group 'helm-ag)
 
 (defcustom helm-ag-use-emacs-lisp-regexp nil
   "[Experimental] Use Emacs Lisp regexp instead of PCRE."
-  :type 'boolean)
+  :type 'boolean
+  :group 'helm-ag)
 
 (defcustom helm-ag-use-agignore nil
   "Use .agignore where is at project root if it exists."
-  :type 'boolean)
+  :type 'boolean
+  :group 'helm-ag)
 
 (defcustom helm-ag-use-temp-buffer nil
   "Use temporary buffer for persistent action."
-  :type 'boolean)
+  :type 'boolean
+  :group 'helm-ag)
 
 (defcustom helm-ag-ignore-buffer-patterns nil
   "Use temporary buffer for persistent action."
-  :type '(repeat regexp))
+  :type '(repeat regexp)
+  :group 'helm-ag)
 
 (defcustom helm-ag-show-status-function 'helm-ag-show-status-default-mode-line
   "Function called after that `ag' process is finished after `helm-do-ag'.
 Default behaviour shows finish and result in mode-line."
-  :type 'function)
+  :type 'function
+  :group 'helm-ag)
 
 (defface helm-ag-edit-deleted-line
   '((t (:inherit font-lock-comment-face :strike-through t)))
@@ -133,6 +146,7 @@ Default behaviour shows finish and result in mode-line."
 (defvar helm-do-ag--commands nil)
 
 (defun helm-ag--ignore-case-p (cmds input)
+  "Not documented."
   (cl-loop for cmd in cmds
            when (member cmd '("-i" "--ignore-case"))
            return t
@@ -145,32 +159,38 @@ Default behaviour shows finish and result in mode-line."
                     (not (string-match-p "[A-Z]" input)))))
 
 (defun helm-ag--save-current-context ()
+  "Not documented."
   (let ((curpoint (with-helm-current-buffer
-                    (point))))
+                   (point))))
     (helm-aif (buffer-file-name helm-current-buffer)
-        (push (list :file it :point curpoint) helm-ag--context-stack)
-      (push (list :buffer helm-current-buffer :point curpoint) helm-ag--context-stack))))
+              (push (list :file it :point curpoint) helm-ag--context-stack)
+              (push (list :buffer helm-current-buffer :point curpoint) helm-ag--context-stack))))
 
 (defun helm-ag--insert-thing-at-point (thing)
+  "Not documented."
   (helm-aif (thing-at-point thing)
-      (substring-no-properties it)
-    ""))
+            (substring-no-properties it)
+            ""))
 
 (defun helm-ag--searched-word ()
+  "Not documented."
   (if helm-ag-insert-at-point
       (helm-ag--insert-thing-at-point helm-ag-insert-at-point)
     ""))
 
 (defun helm-ag--construct-ignore-option (pattern)
+  "Not documented."
   (concat "--ignore=" pattern))
 
 (defun helm-ag--grep-ignore-list-to-options ()
+  "Not documented."
   (require 'grep)
   (cl-loop for ignore in (append grep-find-ignored-files
                                  grep-find-ignored-directories)
            collect (helm-ag--construct-ignore-option ignore)))
 
 (defun helm-ag--parse-options-and-query (input)
+  "Not documented."
   (with-temp-buffer
     (insert input)
     (let (end options)
@@ -178,7 +198,7 @@ Default behaviour shows finish and result in mode-line."
       (when (re-search-forward "\\s-*--\\s-+" nil t)
         (setq end (match-end 0)))
       (goto-char (point-min))
-      (while (re-search-forward "\\(?:^\\|\\s-+\\)\\(-\\S-+\\)\\(?:\\s-+\\|$\\)" end t)
+      (while (re-search-forward "\\(?:\\=\\|\\s-+\\)\\(-\\S-+\\)\\(?:\\s-+\\|$\\)" end t)
         (push (match-string-no-properties 1) options)
         (when end
           (cl-decf end (- (match-end 0) (match-beginning 0))))
@@ -186,6 +206,7 @@ Default behaviour shows finish and result in mode-line."
       (cons options (buffer-string)))))
 
 (defun helm-ag--parse-query (input)
+  "Not documented."
   (let* ((parsed (helm-ag--parse-options-and-query input))
          (options (car parsed))
          (query (cdr parsed)))
@@ -200,10 +221,12 @@ Default behaviour shows finish and result in mode-line."
       (nconc (nreverse options) (list query)))))
 
 (defsubst helm-ag--search-buffer-p (bufname)
+  "Not documented."
   (cl-loop for regexp in helm-ag-ignore-buffer-patterns
            never (string-match-p regexp bufname)))
 
 (defun helm-ag--file-visited-buffers ()
+  "Not documented."
   (let ((bufs (cl-loop for buf in (buffer-list)
                        when (buffer-file-name buf)
                        collect it)))
@@ -214,11 +237,13 @@ Default behaviour shows finish and result in mode-line."
                collect buf))))
 
 (defun helm-ag--construct-targets (targets)
+  "Not documented."
   (let ((default-directory helm-ag--default-directory))
     (cl-loop for target in targets
              collect (file-relative-name target))))
 
 (defun helm-ag--root-agignore ()
+  "Not documented."
   (let ((root (helm-ag--project-root)))
     (when root
       (let ((default-directory root))
@@ -226,6 +251,7 @@ Default behaviour shows finish and result in mode-line."
           (expand-file-name (concat default-directory ".agignore")))))))
 
 (defun helm-ag--construct-command (this-file)
+  "Not documented."
   (let* ((commands (split-string helm-ag-base-command nil t))
          (command (car commands))
          (args (cdr commands)))
@@ -234,7 +260,7 @@ Default behaviour shows finish and result in mode-line."
         (setq args (append args ag-options))))
     (when helm-ag-use-agignore
       (helm-aif (helm-ag--root-agignore)
-          (setq args (append args (list "-p" it)))))
+                (setq args (append args (list "-p" it)))))
     (when helm-ag-ignore-patterns
       (setq args (append args (mapcar 'helm-ag--construct-ignore-option
                                       helm-ag-ignore-patterns))))
@@ -250,6 +276,7 @@ Default behaviour shows finish and result in mode-line."
     (cons command args)))
 
 (defun helm-ag--remove-carrige-returns ()
+  "Not documented."
   (when (helm-ag--windows-p)
     (save-excursion
       (goto-char (point-min))
@@ -257,6 +284,7 @@ Default behaviour shows finish and result in mode-line."
         (replace-match "")))))
 
 (defun helm-ag--abbreviate-file-name ()
+  "Not documented."
   (unless (helm-ag--windows-p)
     (save-excursion
       (goto-char (point-min))
@@ -265,6 +293,7 @@ Default behaviour shows finish and result in mode-line."
         (replace-match (abbreviate-file-name (match-string-no-properties 1)))))))
 
 (defun helm-ag--init ()
+  "Not documented."
   (let ((buf-coding buffer-file-coding-system))
     (helm-attrset 'recenter t)
     (with-current-buffer (helm-candidate-buffer 'global)
@@ -290,12 +319,14 @@ Default behaviour shows finish and result in mode-line."
 (add-to-list 'debug-ignored-errors "^No ag output: ")
 
 (defun helm-ag--search-only-one-file-p ()
+  "Not documented."
   (when (and helm-ag--default-target (= (length helm-ag--default-target) 1))
     (let ((target (car helm-ag--default-target)))
       (unless (file-directory-p target)
         target))))
 
 (defun helm-ag--find-file-action (candidate find-func this-file &optional persistent)
+  "Not documented."
   (when (memq 'pt helm-ag--command-features)
     ;; 'pt' always show filename if matched file is only one.
     (setq this-file nil))
@@ -318,6 +349,7 @@ Default behaviour shows finish and result in mode-line."
            (goto-char (match-beginning 0))))))
 
 (defun helm-ag--open-file-with-temp-buffer (filename)
+  "Not documented."
   (let ((search-directory default-directory))
     (switch-to-buffer (get-buffer-create " *helm-ag persistent*"))
     (setq default-directory search-directory)
@@ -329,15 +361,18 @@ Default behaviour shows finish and result in mode-line."
       (font-lock-fontify-region (point-min) (point-max)))))
 
 (defsubst helm-ag--vimgrep-option ()
+  "Not documented."
   (member "--vimgrep" helm-ag--last-command))
 
 (defun helm-ag--search-this-file-p ()
+  "Not documented."
   (unless (helm-ag--vimgrep-option)
     (if (eq (helm-get-current-source) 'helm-source-do-ag)
         (helm-ag--search-only-one-file-p)
       (helm-attr 'search-this-file))))
 
 (defun helm-ag--persistent-action (candidate)
+  "Not documented."
   (let ((find-func (if helm-ag-use-temp-buffer
                        #'helm-ag--open-file-with-temp-buffer
                      #'find-file)))
@@ -345,6 +380,7 @@ Default behaviour shows finish and result in mode-line."
     (helm-highlight-current-line)))
 
 (defun helm-ag--validate-regexp (regexp)
+  "Not documented."
   (condition-case nil
       (progn
         (string-match-p regexp "")
@@ -352,6 +388,7 @@ Default behaviour shows finish and result in mode-line."
     (invalid-regexp nil)))
 
 (defun helm-ag--pcre-to-elisp-regexp (pcre)
+  "Not documented."
   ;; This is very simple conversion
   (with-temp-buffer
     (insert pcre)
@@ -373,6 +410,7 @@ Default behaviour shows finish and result in mode-line."
     (buffer-string)))
 
 (defun helm-ag--elisp-regexp-to-pcre (regexp)
+  "Not documented."
   (with-temp-buffer
     (insert regexp)
     (goto-char (point-min))
@@ -387,6 +425,7 @@ Default behaviour shows finish and result in mode-line."
     (buffer-string)))
 
 (defun helm-ag--highlight-candidate (candidate)
+  "Not documented."
   (let ((limit (1- (length candidate)))
         (last-pos 0)
         (case-fold-search helm-ag--ignore-case))
@@ -402,28 +441,33 @@ Default behaviour shows finish and result in mode-line."
     candidate))
 
 (defun helm-ag--candidate-transform-for-this-file (candidate)
+  "Not documented."
   (when (string-match "\\`\\([^:]+\\):\\(.*\\)" candidate)
     (format "%s:%s"
             (propertize (match-string 1 candidate) 'face 'helm-grep-lineno)
             (helm-ag--highlight-candidate (match-string 2 candidate)))))
 
 (defun helm-ag--candidate-transform-for-files (candidate)
+  "Not documented."
   (helm-aif (helm-grep-split-line candidate)
-      (format "%s:%s:%s"
-              (propertize (cl-first it) 'face 'helm-moccur-buffer)
-              (propertize (cl-second it) 'face 'helm-grep-lineno)
-              (helm-ag--highlight-candidate (cl-third it)))))
+            (format "%s:%s:%s"
+                    (propertize (cl-first it) 'face 'helm-moccur-buffer)
+                    (propertize (cl-second it) 'face 'helm-grep-lineno)
+                    (helm-ag--highlight-candidate (cl-third it)))))
 
 (defun helm-ag--candidate-transformer (candidate)
+  "Not documented."
   (or (if (helm-attr 'search-this-file)
           (helm-ag--candidate-transform-for-this-file candidate)
         (helm-ag--candidate-transform-for-files candidate))
       candidate))
 
 (defun helm-ag--action-find-file (candidate)
+  "Not documented."
   (helm-ag--find-file-action candidate 'find-file (helm-ag--search-this-file-p)))
 
 (defun helm-ag--action-find-file-other-window (candidate)
+  "Not documented."
   (helm-ag--find-file-action candidate 'find-file-other-window (helm-ag--search-this-file-p)))
 
 (defvar helm-ag--actions
@@ -431,7 +475,8 @@ Default behaviour shows finish and result in mode-line."
    "Open file"              #'helm-ag--action-find-file
    "Open file other window" #'helm-ag--action-find-file-other-window
    "Save results in buffer" #'helm-ag--action-save-buffer
-   "Edit search results"    #'helm-ag--edit))
+   "Edit search results"    #'helm-ag--edit)
+  "Not documented.")
 
 (defvar helm-ag-map
   (let ((map (make-sparse-keymap)))
@@ -450,35 +495,39 @@ Default behaviour shows finish and result in mode-line."
 
 (defvar helm-ag-source
   (helm-build-in-buffer-source "The Silver Searcher"
-    :init 'helm-ag--init
-    :real-to-display 'helm-ag--candidate-transformer
-    :persistent-action 'helm-ag--persistent-action
-    :fuzzy-match helm-ag-fuzzy-match
-    :action helm-ag--actions
-    :candidate-number-limit 9999
-    :keymap helm-ag-map
-    :follow (and helm-follow-mode-persistent 1)))
+                               :init 'helm-ag--init
+                               :real-to-display 'helm-ag--candidate-transformer
+                               :persistent-action 'helm-ag--persistent-action
+                               :fuzzy-match helm-ag-fuzzy-match
+                               :action helm-ag--actions
+                               :candidate-number-limit 9999
+                               :keymap helm-ag-map
+                               :follow (and helm-follow-mode-persistent 1))
+  "Not documented.")
 
 ;;;###autoload
 (defun helm-ag-pop-stack ()
+  "Not documented."
   (interactive)
   (let ((context (pop helm-ag--context-stack)))
     (unless context
       (error "Context stack is empty !"))
     (helm-aif (plist-get context :file)
-        (find-file it)
-      (let ((buf (plist-get context :buffer)))
-        (if (buffer-live-p buf)
-            (switch-to-buffer buf)
-          (error "The buffer is already killed."))))
+              (find-file it)
+              (let ((buf (plist-get context :buffer)))
+                (if (buffer-live-p buf)
+                    (switch-to-buffer buf)
+                  (error "The buffer is already killed."))))
     (goto-char (plist-get context :point))))
 
 ;;;###autoload
 (defun helm-ag-clear-stack ()
+  "Not documented."
   (interactive)
   (setq helm-ag--context-stack nil))
 
 (defun helm-ag--marked-input (escape)
+  "Not documented."
   (when (use-region-p)
     (let ((input (buffer-substring-no-properties (region-beginning) (region-end))))
       (deactivate-mark)
@@ -486,25 +535,29 @@ Default behaviour shows finish and result in mode-line."
           input
         (replace-regexp-in-string " " "\\\\ " input)))))
 
-(defun helm-ag--query ()
+(defun helm-ag--query (&optional query)
+  "Not documented."
   (let* ((searched-word (helm-ag--searched-word))
          (marked-word (helm-ag--marked-input nil))
-         (query (read-from-minibuffer "Pattern: "
-                                      (or marked-word searched-word)
-                                      nil
-                                      nil
-                                      'helm-ag--command-history
-                                      (helm-aif (symbol-at-point)
-                                          (symbol-name it)))))
+         (query (or query
+                    (read-from-minibuffer "Pattern: "
+                                          (or marked-word searched-word)
+                                          nil
+                                          nil
+                                          'helm-ag--command-history
+                                          (helm-aif (symbol-at-point)
+                                                    (symbol-name it))))))
     (when (string-empty-p query)
       (error "Input is empty!!"))
     (setq helm-ag--last-query query)))
 
 (defsubst helm-ag--init-state ()
+  "Not documented."
   (setq helm-ag--original-window (selected-window)
         helm-ag--last-default-directory nil))
 
 (defun helm-ag--get-default-directory ()
+  "Not documented."
   (let ((prefix-val (and current-prefix-arg (abs (prefix-numeric-value current-prefix-arg)))))
     (cond ((not prefix-val) default-directory)
           ((= prefix-val 4)
@@ -517,21 +570,25 @@ Default behaviour shows finish and result in mode-line."
              (reverse dirs))))))
 
 (defsubst helm-ag--helm-header (dir)
+  "Not documented."
   (if helm-ag--buffer-search
       "Search Buffers"
     (concat "Search at " (abbreviate-file-name dir))))
 
 (defun helm-ag--run-other-window-action ()
+  "Not documented."
   (interactive)
   (with-helm-alive-p
-    (helm-exit-and-execute-action #'helm-ag--action-find-file-other-window)))
+   (helm-exit-and-execute-action #'helm-ag--action-find-file-other-window)))
 
 (defun helm-ag--exit-from-edit-mode ()
+  "Not documented."
   (when (window-live-p helm-ag--original-window)
     (select-window helm-ag--original-window))
   (kill-buffer (get-buffer "*helm-ag-edit*")))
 
 (defun helm-ag--match-line-regexp ()
+  "Not documented."
   ;; $1: file name
   ;; $2: line
   ;; $3: match body
@@ -544,6 +601,7 @@ Default behaviour shows finish and result in mode-line."
          "^\\(?4:\\(?1:[^:]+\\):\\(?2:[1-9][0-9]*\\)[:-]\\)\\(?3:.*\\)$")))
 
 (defun helm-ag--edit-commit ()
+  "Not documented."
   (interactive)
   (goto-char (point-min))
   (let ((read-only-files 0)
@@ -587,12 +645,14 @@ Default behaviour shows finish and result in mode-line."
       (message "Success update"))))
 
 (defun helm-ag--edit-abort ()
+  "Not documented."
   (interactive)
   (when (y-or-n-p "Discard changes ?")
     (helm-ag--exit-from-edit-mode)
     (message "Abort edit")))
 
 (defun helm-ag--mark-line-deleted ()
+  "Not documented."
   (interactive)
   (let* ((beg (line-beginning-position))
          (end (line-end-position))
@@ -601,6 +661,7 @@ Default behaviour shows finish and result in mode-line."
     (overlay-put ov 'helm-ag-deleted t)))
 
 (defun helm-ag--unmark ()
+  "Not documented."
   (interactive)
   (dolist (ov (overlays-in (line-beginning-position) (line-end-position)))
     (when (overlay-get ov 'helm-ag-deleted)
@@ -612,18 +673,22 @@ Default behaviour shows finish and result in mode-line."
     (define-key map (kbd "C-c C-k") 'helm-ag--edit-abort)
     (define-key map (kbd "C-c C-d") 'helm-ag--mark-line-deleted)
     (define-key map (kbd "C-c C-u") 'helm-ag--unmark)
-    map))
+    map)
+  "Not documented.")
 
 (defsubst helm-ag--edit-func-to-keys (func)
+  "Not documented."
   (key-description (car-safe (where-is-internal func helm-ag-edit-map))))
 
 (defun helm-ag--edit (_candidate)
+  "Not documented."
   (let* ((helm-buf-dir (or helm-ag--default-directory
                            helm-ag--last-default-directory
                            default-directory))
          (default-directory helm-buf-dir))
     (with-current-buffer (get-buffer-create "*helm-ag-edit*")
-      (erase-buffer)
+      (let ((inhibit-read-only t))
+        (erase-buffer))
       (setq-local helm-ag--default-directory helm-buf-dir)
       (unless (helm-ag--vimgrep-option)
         (setq-local helm-ag--search-this-file-p
@@ -670,6 +735,7 @@ Default behaviour shows finish and result in mode-line."
   (use-local-map helm-ag-edit-map))
 
 (defun helm-ag-edit ()
+  "Not documented."
   (interactive)
   (helm-exit-and-execute-action 'helm-ag--edit))
 
@@ -683,19 +749,23 @@ Default behaviour shows finish and result in mode-line."
 \\[helm-ag-edit]\t\t-> Edit search results.
 \\[helm-ag-help]\t\t-> Show this help.
 \n** Helm Ag Map\n
-\\{helm-map}")
+\\{helm-map}"
+  "Not documented.")
 
 (defun helm-ag-help ()
+  "Not documented."
   (interactive)
   (let ((helm-help-message helm-ag--help-message))
     (helm-help)))
 
 (defun helm-ag-mode-jump ()
+  "Not documented."
   (interactive)
   (let ((line (helm-current-line-contents)))
     (helm-ag--find-file-action line 'find-file helm-ag--search-this-file-p)))
 
 (defun helm-ag-mode-jump-other-window ()
+  "Not documented."
   (interactive)
   (let ((line (helm-current-line-contents)))
     (helm-ag--find-file-action line 'find-file-other-window helm-ag--search-this-file-p)))
@@ -705,7 +775,8 @@ Default behaviour shows finish and result in mode-line."
     (define-key map (kbd "RET") 'helm-ag-mode-jump)
     (define-key map (kbd "C-o") 'helm-ag-mode-jump-other-window)
     (define-key map (kbd "g") 'helm-ag--update-save-results)
-    map))
+    map)
+  "Not documented.")
 
 (define-derived-mode helm-ag-mode special-mode "helm-ag"
   "Major mode to provide actions in helm grep saved buffer.
@@ -714,6 +785,7 @@ Special commands:
 \\{helm-ag-mode-map}")
 
 (defun helm-ag--put-result-in-save-buffer (result search-this-file-p)
+  "Not documented."
   (setq buffer-read-only t)
   (let ((inhibit-read-only t))
     (erase-buffer)
@@ -727,6 +799,7 @@ Special commands:
   (setq-local helm-ag--default-directory default-directory))
 
 (defun helm-ag--save-results (use-other-buf)
+  "Not documented."
   (let* ((search-this-file-p nil)
          (result (with-current-buffer helm-buffer
                    (goto-char (point-min))
@@ -745,6 +818,7 @@ Special commands:
       (message "Helm Ag Results saved in `%s' buffer" buf))))
 
 (defun helm-ag--update-save-results ()
+  "Not documented."
   (interactive)
   (let* ((default-directory helm-ag--default-directory)
          (result (with-temp-buffer
@@ -759,45 +833,53 @@ Special commands:
     (message "Update Results")))
 
 (defun helm-ag--action-save-buffer (_arg)
+  "Not documented."
   (helm-ag--save-results nil))
 
 (defun helm-ag--run-save-buffer ()
+  "Not documented."
   (interactive)
   (let ((use-other-buf-p current-prefix-arg))
     (with-helm-alive-p
-      (helm-exit-and-execute-action
-       (lambda (_arg)
-         (helm-ag--save-results use-other-buf-p))))))
+     (helm-exit-and-execute-action
+      (lambda (_arg)
+        (helm-ag--save-results use-other-buf-p))))))
 
 (defun helm-ag--file-of-current-file ()
+  "Not documented."
   (let ((line (helm-current-line-contents)))
     (when (string-match helm-grep-split-line-regexp line)
       (match-string-no-properties 1 line))))
 
 (defun helm-ag--move-file-common (pred move-fn wrap-fn)
+  "Not documented."
   (with-helm-window
-    (let ((file (helm-ag--file-of-current-file)))
-      (funcall move-fn)
-      (while (and (not (funcall pred)) (string= file (helm-ag--file-of-current-file)))
-        (funcall move-fn))
-      (when (funcall pred)
-        (funcall wrap-fn)))))
+   (let ((file (helm-ag--file-of-current-file)))
+     (funcall move-fn)
+     (while (and (not (funcall pred)) (string= file (helm-ag--file-of-current-file)))
+       (funcall move-fn))
+     (when (funcall pred)
+       (funcall wrap-fn)))))
 
 (defun helm-ag--previous-file ()
+  "Not documented."
   (interactive)
   (helm-ag--move-file-common
    #'helm-beginning-of-source-p #'helm-previous-line #'helm-end-of-buffer))
 
 (defun helm-ag--next-file ()
+  "Not documented."
   (interactive)
   (helm-ag--move-file-common
    #'helm-end-of-source-p #'helm-next-line #'helm-beginning-of-buffer))
 
 (defsubst helm-ag--root-directory-p ()
+  "Not documented."
   (cl-loop for dir in '(".git/" ".hg/")
            thereis (file-directory-p dir)))
 
 (defun helm-ag--up-one-level ()
+  "Not documented."
   (interactive)
   (if (or (not (helm-ag--root-directory-p))
           (y-or-n-p "Current directory might be the project root. \
@@ -814,12 +896,13 @@ Continue searching the parent directory? "))
     (message nil)))
 
 ;;;###autoload
-(defun helm-ag-this-file ()
+(defun helm-ag-this-file (&optional query)
+  "Do ag with in this file with QUERY."
   (interactive)
   (helm-ag--init-state)
   (let ((filename (file-name-nondirectory (buffer-file-name)))
         (helm-ag--default-directory default-directory))
-    (helm-ag--query)
+    (helm-ag--query query)
     (helm-ag--set-command-features)
     (helm-attrset 'search-this-file (file-relative-name (buffer-file-name))
                   helm-ag-source)
@@ -828,7 +911,8 @@ Continue searching the parent directory? "))
           :history 'helm-ag--helm-history)))
 
 ;;;###autoload
-(defun helm-ag (&optional basedir)
+(defun helm-ag (&optional basedir query)
+  "Do ag with in BASEDIR and with QUERY."
   (interactive)
   (helm-ag--init-state)
   (let ((dir (helm-ag--get-default-directory))
@@ -838,13 +922,14 @@ Continue searching the parent directory? "))
             targets dir))
     (let ((helm-ag--default-directory (or basedir dir))
           (helm-ag--default-target targets))
-      (helm-ag--query)
+      (helm-ag--query query)
       (helm-attrset 'search-this-file nil helm-ag-source)
       (helm-attrset 'name (helm-ag--helm-header helm-ag--default-directory) helm-ag-source)
       (helm :sources '(helm-ag-source) :buffer "*helm-ag*" :keymap helm-ag-map
             :history 'helm-ag--helm-history))))
 
 (defun helm-ag--split-string (str)
+  "Not documented."
   (with-temp-buffer
     (insert str)
     (goto-char (point-min))
@@ -864,11 +949,13 @@ Continue searching the parent directory? "))
       (reverse (cl-loop for p in patterns unless (string= p "") collect p)))))
 
 (defsubst helm-ag--convert-invert-pattern (pattern)
+  "Not documented."
   (when (and (memq 'pcre helm-ag--command-features)
              (string-prefix-p "!" pattern) (> (length pattern) 1))
     (concat "^(?!.*" (substring pattern 1) ").+$")))
 
 (defun helm-ag--join-patterns (input)
+  "Not documented."
   (let ((patterns (helm-ag--split-string input)))
     (if (= (length patterns) 1)
         (or (helm-ag--convert-invert-pattern (car patterns))
@@ -885,6 +972,7 @@ Continue searching the parent directory? "))
             (t input)))))
 
 (defun helm-ag--do-ag-highlight-patterns (input)
+  "Not documented."
   (if (memq 'pcre helm-ag--command-features)
       (cl-loop with regexp = (helm-ag--pcre-to-elisp-regexp input)
                for pattern in (helm-ag--split-string regexp)
@@ -893,6 +981,7 @@ Continue searching the parent directory? "))
     (list (helm-ag--join-patterns input))))
 
 (defun helm-ag--propertize-candidates (input)
+  "Not documented."
   (save-excursion
     (goto-char (point-min))
     (forward-line 1)
@@ -927,6 +1016,7 @@ Continue searching the parent directory? "))
                  (forward-line 1))))))
 
 (defun helm-ag-show-status-default-mode-line ()
+  "Not documented."
   (setq mode-line-format
         '(" " mode-line-buffer-identification " "
           (:eval (propertize
@@ -936,16 +1026,18 @@ Continue searching the parent directory? "))
                   'face 'helm-grep-finish)))))
 
 (defun helm-ag--do-ag-propertize (input)
+  "Not documented."
   (with-helm-window
-    (helm-ag--remove-carrige-returns)
-    (when helm-ag--buffer-search
-      (helm-ag--abbreviate-file-name))
-    (helm-ag--propertize-candidates input)
-    (when helm-ag-show-status-function
-      (funcall helm-ag-show-status-function)
-      (force-mode-line-update))))
+   (helm-ag--remove-carrige-returns)
+   (when helm-ag--buffer-search
+     (helm-ag--abbreviate-file-name))
+   (helm-ag--propertize-candidates input)
+   (when helm-ag-show-status-function
+     (funcall helm-ag-show-status-function)
+     (force-mode-line-update))))
 
 (defun helm-ag--construct-extension-options ()
+  "Not documented."
   (cl-loop for ext in helm-do-ag--extensions
            unless (string= ext "*")
            collect
@@ -954,11 +1046,13 @@ Continue searching the parent directory? "))
                          (replace-regexp-in-string "\\." "\\\\." ext)))))
 
 (defun helm-ag--show-result-p (options has-query)
+  "Not documented."
   (or has-query
       (cl-loop for opt in options
                thereis (string-prefix-p "-g" opt))))
 
 (defun helm-ag--construct-do-ag-command (pattern)
+  "Not documented."
   (let* ((opt-query (helm-ag--parse-options-and-query pattern))
          (options (car opt-query))
          (query (cdr opt-query))
@@ -972,6 +1066,7 @@ Continue searching the parent directory? "))
               (cdr helm-do-ag--commands)))))
 
 (defun helm-ag--do-ag-set-command ()
+  "Not documented."
   (let ((cmd-opts (split-string helm-ag-base-command nil t)))
     (when helm-ag-command-option
       (setq cmd-opts (append cmd-opts (split-string helm-ag-command-option nil t))))
@@ -984,7 +1079,7 @@ Continue searching the parent directory? "))
                             helm-ag-ignore-patterns))))
     (when helm-ag-use-agignore
       (helm-aif (helm-ag--root-agignore)
-          (setq cmd-opts (append cmd-opts (list "-p" it)))))
+                (setq cmd-opts (append cmd-opts (list "-p" it)))))
     (when helm-do-ag--extensions
       (setq cmd-opts (append cmd-opts (helm-ag--construct-extension-options))))
     (let (targets)
@@ -997,6 +1092,7 @@ Continue searching the parent directory? "))
                     targets))))))
 
 (defun helm-ag--do-ag-candidate-process ()
+  "Not documented."
   (let* ((non-essential nil)
          (default-directory (or helm-ag--default-directory
                                 helm-ag--last-default-directory
@@ -1027,9 +1123,11 @@ Continue searching the parent directory? "))
 \\[helm-ag-edit]\t\t-> Edit search results.
 \\[helm-ag--do-ag-help]\t\t-> Show this help.
 \n** Helm Ag Map\n
-\\{helm-map}")
+\\{helm-map}"
+  "Not documented.")
 
 (defun helm-ag--do-ag-help ()
+  "Not documented."
   (interactive)
   (let ((helm-help-message helm-do-ag--help-message))
     (helm-help)))
@@ -1044,17 +1142,19 @@ Continue searching the parent directory? "))
 
 (defvar helm-source-do-ag
   (helm-build-async-source "The Silver Searcher"
-    :init 'helm-ag--do-ag-set-command
-    :candidates-process 'helm-ag--do-ag-candidate-process
-    :persistent-action  'helm-ag--persistent-action
-    :action helm-ag--actions
-    :nohighlight t
-    :requires-pattern 3
-    :candidate-number-limit 9999
-    :keymap helm-do-ag-map
-    :follow (and helm-follow-mode-persistent 1)))
+                           :init 'helm-ag--do-ag-set-command
+                           :candidates-process 'helm-ag--do-ag-candidate-process
+                           :persistent-action  'helm-ag--persistent-action
+                           :action helm-ag--actions
+                           :nohighlight t
+                           :requires-pattern 3
+                           :candidate-number-limit 9999
+                           :keymap helm-do-ag-map
+                           :follow (and helm-follow-mode-persistent 1))
+  "Not documented.")
 
 (defun helm-ag--do-ag-up-one-level ()
+  "Not documented."
   (interactive)
   (if (or (not (helm-ag--root-directory-p))
           (y-or-n-p "Current directory might be the project root. \
@@ -1074,6 +1174,7 @@ Continue searching the parent directory? "))
     (message nil)))
 
 (defun helm-ag--set-do-ag-option ()
+  "Not documented."
   (when (or (< (prefix-numeric-value current-prefix-arg) 0)
             helm-ag-always-set-extra-option)
     (let ((option (read-string "Extra options: " (or helm-ag--extra-options "")
@@ -1081,6 +1182,7 @@ Continue searching the parent directory? "))
       (setq helm-ag--extra-options option))))
 
 (defun helm-ag--set-command-features ()
+  "Not documented."
   (let ((cmd (intern (car (split-string helm-ag-base-command)))))
     (setq helm-ag--command-features (list cmd))
     (cl-case cmd
@@ -1102,13 +1204,16 @@ Continue searching the parent directory? "))
                          're2))))))
 
 (defun helm-ag--do-ag-searched-extensions ()
+  "Not documented."
   (when (and current-prefix-arg (= (abs (prefix-numeric-value current-prefix-arg)) 4))
     (helm-grep-get-file-extensions helm-ag--default-target)))
 
 (defsubst helm-do-ag--target-one-directory-p (targets)
+  "Not documented."
   (and (listp targets) (= (length targets) 1) (file-directory-p (car targets))))
 
-(defun helm-do-ag--helm ()
+(defun helm-do-ag--helm (&optional query)
+  "Not documented."
   (let ((search-dir (if (not (helm-ag--windows-p))
                         helm-ag--default-directory
                       (if (helm-do-ag--target-one-directory-p helm-ag--default-target)
@@ -1117,19 +1222,25 @@ Continue searching the parent directory? "))
     (helm-attrset 'name (helm-ag--helm-header search-dir)
                   helm-source-do-ag)
     (helm :sources '(helm-source-do-ag) :buffer "*helm-ag*" :keymap helm-do-ag-map
-          :input (or (helm-ag--marked-input t)
-                     (helm-ag--insert-thing-at-point helm-ag-insert-at-point))
+          :input (let ((input (or query
+                                  (helm-ag--marked-input t)
+                                  (helm-ag--insert-thing-at-point helm-ag-insert-at-point))))
+                   (if (string-prefix-p "-" input)
+                       (concat "-- " input)
+                     input))
           :history 'helm-ag--helm-history)))
 
 ;;;###autoload
-(defun helm-do-ag-this-file ()
+(defun helm-do-ag-this-file (&optional query)
+  "Not documented."
   (interactive)
   (helm-aif (buffer-file-name)
-      (helm-do-ag default-directory (list it))
-    (error "Error: This buffer is not visited file.")))
+            (helm-do-ag default-directory (list it) query)
+            (error "Error: This buffer is not visited file.")))
 
 ;;;###autoload
-(defun helm-do-ag (&optional basedir targets)
+(defun helm-do-ag (&optional basedir targets query)
+  "Not documented."
   (interactive)
   (require 'helm-mode)
   (helm-ag--init-state)
@@ -1155,44 +1266,49 @@ Continue searching the parent directory? "))
                        (car helm-ag--default-target))
                   helm-source-do-ag)
     (if (or (helm-ag--windows-p) (not one-directory-p)) ;; Path argument must be specified on Windows
-        (helm-do-ag--helm)
+        (helm-do-ag--helm query)
       (let* ((helm-ag--default-directory
               (file-name-as-directory (car helm-ag--default-target)))
              (helm-ag--default-target nil))
-        (helm-do-ag--helm)))))
+        (helm-do-ag--helm query)))))
 
 (defun helm-ag--project-root ()
+  "Not documented."
   (cl-loop for dir in '(".git/" ".hg/" ".svn/" ".git")
            when (locate-dominating-file default-directory dir)
            return it))
 
 ;;;###autoload
-(defun helm-ag-project-root ()
+(defun helm-ag-project-root (&optional query)
+  "Not documented."
   (interactive)
   (let ((rootdir (helm-ag--project-root)))
     (unless rootdir
       (error "Could not find the project root. Create a git, hg, or svn repository there first. "))
-    (helm-ag rootdir)))
+    (helm-ag rootdir query)))
 
 ;;;###autoload
-(defun helm-do-ag-project-root ()
+(defun helm-do-ag-project-root (&optional query)
+  "Not documented."
   (interactive)
   (let ((rootdir (helm-ag--project-root)))
     (unless rootdir
       (error "Could not find the project root. Create a git, hg, or svn repository there first. "))
-    (helm-do-ag rootdir)))
+    (helm-do-ag rootdir nil query)))
 
 ;;;###autoload
-(defun helm-ag-buffers ()
+(defun helm-ag-buffers (&optional query)
+  "Not documented."
   (interactive)
   (let ((helm-ag--buffer-search t))
-    (helm-ag)))
+    (helm-ag nil query)))
 
 ;;;###autoload
-(defun helm-do-ag-buffers ()
+(defun helm-do-ag-buffers (&optional query)
+  "Not documented."
   (interactive)
   (let ((helm-ag--buffer-search t))
-    (helm-do-ag)))
+    (helm-do-ag nil nil query)))
 
 (provide 'helm-ag)
 
